enum UserInfoStateEnum {
	Nonactivated,
	Activated,
}

class UserInfo {
	String? userCode;
	String? userName;
	String? phone;
	String? email;
	String? nickName;
	String? fullName;
	String? headImageToken;
	String? organizationCode;
	List<String>? authorityGroups;
	List<String>? bindDevices;
	int score;
	String? lastIP;
	UserInfoStateEnum userState;
	String? securityQuestion;
	String? securityAnswers;
	DateTime? createTime;
	DateTime? updateTime;

	UserInfo({
		this.userCode,
		this.userName,
		this.phone,
		this.email,
		this.nickName,
		this.fullName,
		this.headImageToken,
		this.organizationCode,
		this.authorityGroups,
		this.bindDevices,
		this.score=0,
		this.lastIP,
		this.userState=UserInfoStateEnum.Nonactivated,
		this.securityQuestion,
		this.securityAnswers,
		this.createTime,
		this.updateTime,
	});

	factory UserInfo.fromJson(Map<String, dynamic> map) {
		return UserInfo( 
			userCode: map['UserCode'],
			userName: map['UserName'],
			phone: map['Phone'],
			email: map['Email'],
			nickName: map['NickName'],
			fullName: map['FullName'],
			headImageToken: map['HeadImageToken'],
			organizationCode: map['OrganizationCode'],
			authorityGroups: map['AuthorityGroups'].cast<String>().toList(),
			bindDevices: map['BindDevices'].cast<String>().toList(),
			score: map['Score'],
			lastIP: map['LastIP'],
			userState: UserInfoStateEnum.values.firstWhere((e) => e.index == map['UserState']),
			securityQuestion: map['SecurityQuestion'],
			securityAnswers: map['SecurityAnswers'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(userCode != null)
			map['UserCode'] = userCode;
		if(userName != null)
			map['UserName'] = userName;
		if(phone != null)
			map['Phone'] = phone;
		if(email != null)
			map['Email'] = email;
		if(nickName != null)
			map['NickName'] = nickName;
		if(fullName != null)
			map['FullName'] = fullName;
		if(headImageToken != null)
			map['HeadImageToken'] = headImageToken;
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		if(authorityGroups != null)
			map['AuthorityGroups'] = authorityGroups;
		if(bindDevices != null)
			map['BindDevices'] = bindDevices;
		map['Score'] = score;
		if(lastIP != null)
			map['LastIP'] = lastIP;
		map['UserState'] = userState.index;
		if(securityQuestion != null)
			map['SecurityQuestion'] = securityQuestion;
		if(securityAnswers != null)
			map['SecurityAnswers'] = securityAnswers;
		if(createTime != null)
			map['CreateTime'] = createTime;
		if(updateTime != null)
			map['UpdateTime'] = updateTime;
		return map;
	}
}


